@@ -1,6 +1,6 @@
 from scalaccf import utility
 from scalaccf.ccf import CCF
-
+from scalaccf.rename_dirs import rename_dirs
 
 def main():
     argument_parser = utility.setup_argument_parser()
@@ -18,3 +18,6 @@ def main():
     elif mode == 'trace':
         # fix file and dirs namings
         ccf.fix()
+
+    if path_arg == 'p':
+        rename_dirs(args.p, mode)
