@@ -11,6 +11,8 @@ class CCF:
     def verify(self):
         if not os.path.exists('verification'):
             os.mkdir('verification')
+        logging.basicConfig(filename=os.path.join('verification', 'verification.log'),
+                            level=logging.WARN)
         tokens_list = []
         for file in self.__files:
             tokens_list.append(Lexer(file).get_tokens())
@@ -19,8 +21,6 @@ class CCF:
         while i < len(tokens_list):
             tokens = tokens_list[i]
             file = self.__files[i]
-            logging.basicConfig(filename=os.path.join('verification', f'{file.split(".")[0]}_verification.log'),
-                                level=logging.WARN)
             formatter = Formatter(tokens)
             expected_file_name = formatter.fix_filename(file)
             if expected_file_name is not None and expected_file_name != file:
@@ -45,6 +45,7 @@ class CCF:
     def fix(self):
         if not os.path.exists('fixing'):
             os.mkdir('fixing')
+        logging.basicConfig(filename=os.path.join('fixing', 'fixing.log'), level=logging.WARN)
         tokens_list = []
         for file in self.__files:
             tokens_list.append(Lexer(file).get_tokens())
@@ -53,7 +54,6 @@ class CCF:
         while i < len(tokens_list):
             tokens = tokens_list[i]
             file = self.__files[i]
-            logging.basicConfig(filename=os.path.join('fixing', f'{file.split(".")[0]}_fixing.log'), level=logging.WARN)
             formatter = Formatter(tokens)
             expected_file_name = formatter.fix_filename(file)
             if expected_file_name is not None and expected_file_name != file:
