@@ -42,8 +42,12 @@ class Formatter:
         lower_camel_case = Formatter.snake_to_lower_camel_case(naming)
         return lower_camel_case[0].upper() + lower_camel_case[1:]
 
+    @staticmethod
+    def to_lower(naming):
+        return ''.join([component.lower() for component in naming.split('_')])
+
     def fix_namings(self, namings_to_fix):
-        self.__fix_comments()
+        self.__fix_comments(namings_to_fix)
         self.__fix_namings(namings_to_fix)
 
     def fix_filename(self, filename):
@@ -77,8 +81,6 @@ class Formatter:
             if token.get_value() == 'package':
                 i = self.__fix_packages(i) - 1
             # upper case camel case for class, trait & object
-            elif token.get_value() == 'extends':
-                i = self.__fix_class_trait_object(i) - 1
             elif token.get_value() in Formatter.class_keywords:
                 if token.get_value() == 'object' and self.__tokens[i - 2].get_value() == 'package':
                     i = self.__fix_packages(i) - 1
@@ -104,11 +106,12 @@ class Formatter:
             token = self.__tokens[pos]
             if token.get_token_type() == TokenType.IDENTIFIER and Formatter.__is_not_lower_camel_case(
                     token.get_value()):
-                token.set_fixed_value(Formatter.snake_to_lower_camel_case(token.get_value()))
+                token.set_fixed_value(Formatter.to_lower(token.get_value()))
             pos += 1
         return pos
 
     def __fix_class_trait_object(self, pos):
+        self.__fix_class_params(pos)
         pos = self.__skip_ws_tokens(pos + 1)
         if self.__tokens[pos].get_token_type() == TokenType.IDENTIFIER:
             if Formatter.__is_not_upper_camel_case(
@@ -116,7 +119,6 @@ class Formatter:
                 fixed_value = Formatter.snake_to_upper_camel_case(self.__tokens[pos].get_value())
                 self.__namings_to_fix[self.__tokens[pos].get_primary_value()] = fixed_value
             pos = self.__fix_generic_if_exists(pos)
-            self.__fix_class_params(pos)
         return pos
 
     def __fix_generic_if_exists(self, pos):
@@ -129,7 +131,7 @@ class Formatter:
                 pos += 1
         return pos
 
-    def __fix_comments(self):
+    def __fix_comments(self, namings_to_fix):
         # Fix documentation comments
         i = 0
         while i < len(self.__tokens):
@@ -140,15 +142,15 @@ class Formatter:
                 if pos != -1:
                     comment = self.__tokens[pos]
                 if token.get_value() == 'def':
-                    self.__fix_method_comment(i, comment)
+                    self.__fix_method_comment(i, namings_to_fix, comment)
                 elif token.get_value() == 'package':
-                    self.__fix_package_comment(i, comment)
+                    self.__fix_package_comment(i, namings_to_fix, comment)
                 elif token.get_value() == 'class':
-                    self.__fix_class_comment(i, comment)
+                    self.__fix_class_comment(i, namings_to_fix, comment)
                 elif token.get_value() == 'trait':
                     self.__fix_trait_comment(i, comment)
                 elif token.get_value() == 'object':
-                    self.__fix_object_comment(i, comment)
+                    self.__fix_object_comment(i, namings_to_fix, comment)
                 i += 1
             i += 1
 
@@ -184,7 +186,7 @@ class Formatter:
             pos -= 1
         return last_throws
 
-    def __fix_package_comment(self, pos, comment=None):
+    def __fix_package_comment(self, pos, namings_to_fix, comment=None):
         if comment is not None:
             return
         ws = ''
@@ -200,16 +202,12 @@ class Formatter:
         if len(classes) == 0:
             return
         comment = ws + '/** '
-        package = self.find_package()
-        if Formatter.__is_not_lower_camel_case(package) and Formatter.__is_not_lower_camel_case(package):
-            package = Formatter.snake_to_lower_camel_case(package)
-        components = package.split('.')
+        components = self.find_package()
         package = '.'.join(
-            Formatter.snake_to_lower_camel_case(component) if Formatter.__is_not_lower_camel_case(
-                component) else comment for
+            namings_to_fix[component] if comment in namings_to_fix.keys() else comment for
             component in components)
         for i in classes:
-            comment += f'Class implemented in this package is [[{package}.{self.snake_to_upper_camel_case(i) if Formatter.__is_not_upper_camel_case(i) else i}]]\n'
+            comment += f'Class implemented in this package is [[{package}.{namings_to_fix[i] if i in namings_to_fix else i}]]\n'
         comment += ws + '*/\n' + ws
 
         token = Token(None, TokenType.COMMENT, self.__tokens[pos].get_position())
@@ -240,9 +238,10 @@ class Formatter:
         package = list()
         while i < len(self.__tokens):
             token = self.__tokens[i]
-            if token.get_value() == 'package':
+            if token.get_value() == 'package' and not package_found:
                 package_found = True
-            elif token.get_value() in ('class', 'trait', 'object', 'import'):
+            elif token.get_value() in ('class', 'trait', 'object', 'import') or (
+                    token.get_value() == 'package' and package_found):
                 return package
             elif package_found and \
                     token.get_token_type() == TokenType.IDENTIFIER:
@@ -302,7 +301,7 @@ class Formatter:
                 return True
         return False
 
-    def __fix_class_comment(self, pos, comment=None):
+    def __fix_class_comment(self, pos, namings_to_fix, comment=None):
         name = self.__find_class_name(pos)
         type_params = self.__find_type_params(pos)
         params = self.__find_params(pos)
@@ -310,10 +309,14 @@ class Formatter:
         extends = self.__find_extends(pos)
 
         if comment is not None:
-            comment.set_fixed_value(Formatter.__replace_in_comment('@tparam', type_params, comment.get_value(), True))
-            comment.set_fixed_value(Formatter.__replace_in_comment('@param', params, comment.get_value(), False))
-            comment.set_fixed_value(Formatter.__replace_in_comment('@throws', throws, comment.get_value(), True))
-            comment.set_fixed_value(Formatter.__replace_in_comment('@extends', extends, comment.get_value(), True))
+            comment.set_fixed_value(
+                Formatter.__replace_in_comment('@tparam', type_params, comment.get_value(), namings_to_fix))
+            comment.set_fixed_value(
+                Formatter.__replace_in_comment('@param', params, comment.get_value(), namings_to_fix))
+            comment.set_fixed_value(
+                Formatter.__replace_in_comment('@throws', throws, comment.get_value(), namings_to_fix))
+            comment.set_fixed_value(
+                Formatter.__replace_in_comment('@extends', extends, comment.get_value(), namings_to_fix))
 
             comment.set_fixed_value(
                 Formatter.__add_to_comment_if_not_exists('@tparam', type_params, comment.get_value(), True))
@@ -360,11 +363,10 @@ class Formatter:
             comment += ws + '*/\n'
             token = Token(None, TokenType.COMMENT, self.__tokens[pos].get_position())
             token.set_fixed_value(comment)
-            print(pos)
             self.__tokens.insert(pos, token)
 
     @staticmethod
-    def __replace_in_comment(param, tokens, comment, upper):
+    def __replace_in_comment(param, tokens, comment, namings_to_fix):
         pos = 0
         while pos < len(comment):
             pos = comment.find(param, pos)
@@ -393,13 +395,8 @@ class Formatter:
                 token_end = len(comment) - 1
 
             token = comment[token_start:token_end]
-            if token in tokens or Formatter.snake_to_lower_camel_case(
-                    token) in tokens or Formatter.snake_to_upper_camel_case(token) in tokens:
-                if upper and Formatter.__is_not_upper_camel_case(token):
-                    token = Formatter.snake_to_upper_camel_case(token)
-                elif not upper and Formatter.__is_not_lower_camel_case(token):
-                    token = Formatter.snake_to_lower_camel_case(token)
-            comment = comment[:token_start] + token + comment[token_end:]
+            if token in tokens and token in namings_to_fix.keys():
+                comment = comment.replace(token, namings_to_fix[token], token_start)
             pos = token_end + 1
         return comment
 
@@ -454,9 +451,11 @@ class Formatter:
         params = self.__find_params(pos)
         brace_found = False
         brace_count = 0
+        parenthesis_count = 0
         while not (brace_found and brace_count == 0):
             if self.__tokens[pos].get_value() in params and Formatter.__is_not_lower_camel_case(
                     self.__tokens[pos].get_value()):
+                print(self.__tokens[pos])
                 self.__tokens[pos].set_fixed_value(
                     Formatter.snake_to_lower_camel_case(self.__tokens[pos].get_value()))
             elif self.__tokens[pos].get_value() == '{':
@@ -464,7 +463,11 @@ class Formatter:
                 brace_count += 1
             elif self.__tokens[pos].get_value() == '}':
                 brace_count -= 1
-            elif self.__tokens[pos].get_value() in ['def', 'val', 'var']:
+            elif self.__tokens[pos].get_value() == '(':
+                parenthesis_count += 1
+            elif self.__tokens[pos].get_value() == ')':
+                parenthesis_count -= 1
+            elif self.__tokens[pos].get_value() in ['def', 'val', 'var'] and parenthesis_count == 0:
                 current = self.__tokens[pos].get_value()
                 pos = self.__skip_ws_tokens(pos + 1)
                 if current == 'val' and self.__tokens[pos].get_fixed_value() is None:
@@ -477,10 +480,10 @@ class Formatter:
                             self.__tokens[pos].get_value())
             pos += 1
 
-    def __fix_method_comment(self, pos, comment=None):
+    def __fix_method_comment(self, pos, namings_to_fix, comment=None):
         params = self.__find_params(pos)
         throws = self.__find_throws(pos)
-        (returns_before, returns) = self.__find_and_format_returns(pos)
+        returns = self.__find_returns(pos)
 
         ws = ''
         i = pos - 1
@@ -493,23 +496,19 @@ class Formatter:
             i += 1
 
         if comment is not None:
-            comment.set_fixed_value(Formatter.__replace_in_comment('@param', params, comment.get_value(), False))
-            comment.set_fixed_value(Formatter.__replace_in_comment('@throws', throws, comment.get_value(), True))
             comment.set_fixed_value(
-                Formatter.__replace_in_comment('@return', returns_before, comment.get_value(), True))
-            if len(returns_before) > 0:
+                Formatter.__replace_in_comment('@param', params, comment.get_value(), namings_to_fix))
+            comment.set_fixed_value(
+                Formatter.__replace_in_comment('@throws', throws, comment.get_value(), namings_to_fix))
+            if len(returns) > 0:
                 comment.set_fixed_value(
-                    Formatter.__replace_in_comment('@return', Formatter.snake_to_lower_camel_case(
-                        returns_before[0].lower() + returns_before[1:]),
-                                                   comment.get_value(), True))
-            comment.set_fixed_value(
-                Formatter.__add_to_comment_if_not_exists('@param', params, comment.get_value(), False, ws))
-            comment.set_fixed_value(
-                Formatter.__add_to_comment_if_not_exists('@throws', throws, comment.get_value(), True, ws))
+                    Formatter.__replace_in_comment('@return', Formatter.__fix_return(returns, namings_to_fix),
+                                                   comment.get_value(), namings_to_fix))
             if returns != '':
                 comment.set_fixed_value(
                     Formatter.__add_to_comment_if_not_exists('@return', [returns], comment.get_value(), True, ws))
         else:
+            comment = ''
             params = [
                 Formatter.snake_to_lower_camel_case(param) if Formatter.__is_not_lower_camel_case(param) else param
                 for param in params]
@@ -527,7 +526,7 @@ class Formatter:
                 elif len(throws) > 0:
                     comment = '\n' + ws + f'/** @throws {throws[0]}\n'
                     i = 1
-                else:
+                elif len(returns) > 0:
                     comment = '\n' + ws + f'/** @return {returns}\n'
                 if len(throws) > 0:
                     while i < len(throws):
@@ -547,25 +546,43 @@ class Formatter:
             pos += 1
         return self.__tokens[self.__skip_ws_tokens(pos + 1)].get_value()
 
-    def __find_and_format_returns(self, pos):
-        brace_found = False
+    @staticmethod
+    def __fix_return(returns, namings_to_fix):
+        for naming in namings_to_fix.keys():
+            pos = returns.find(naming)
+            if pos != -1:
+                if returns[pos - 1] in ('[', ',', ' ', ':') and returns[pos + len(naming) + 1] in (
+                        ']', ',', ' ', ':'):
+                    returns = returns.replace(naming, namings_to_fix[naming], pos)
+
+        return returns
+
+    def __find_returns(self, pos):
+        brace_count = 0
+        square_count = 0
         colon_found = False
-        returns_before = ''
         returns = ''
         pos += 1
         while self.__tokens[pos].get_value() not in ('{', '=', 'val', 'def') and \
-                self.__tokens[pos].get_value() not in Formatter.class_keywords:
+                self.__tokens[pos].get_value() not in Formatter.class_keywords and not \
+                (self.__tokens[pos].get_token_type() == TokenType.WHITESPACE and square_count == 0 and len(
+                    returns) != 0 and colon_found):
             token = self.__tokens[pos]
-            if token.get_value() == ')':
-                brace_found = True
-            elif brace_found and token.get_value() == ':':
+            if token.get_value() == '(':
+                brace_count += 1
+            elif token.get_value() == ')':
+                brace_count -= 1
+            elif brace_count == 0 and square_count == 0 and token.get_value() == ':':
                 colon_found = True
-            elif colon_found and token.get_token_type() == TokenType.IDENTIFIER or token.get_value() == (',', '[', ']'):
-                returns_before += token.get_value()
-                returns += self.snake_to_upper_camel_case(token.get_value()) if Formatter.__is_not_upper_camel_case(
-                    token.get_value()) else token.get_value()
+            elif colon_found and (token.get_token_type() == TokenType.IDENTIFIER or token.get_value() in (
+                    ',', '[', ']')):
+                returns += token.get_value()
+                if token.get_value() == '[':
+                    square_count += 1
+                elif token.get_value() == ']':
+                    square_count -= 1
             pos += 1
-        return returns_before, returns
+        return returns
 
     def __find_params(self, pos):
         params = []
@@ -610,10 +627,11 @@ class Formatter:
             pos += 1
         return throws
 
-    def __fix_object_comment(self, pos, comment=None):
+    def __fix_object_comment(self, pos, namings_to_fix, comment=None):
         extends = self.__find_extends(pos)
         if comment is not None:
-            comment.set_fixed_value(Formatter.__replace_in_comment('@extends', extends, comment.get_value(), True))
+            comment.set_fixed_value(
+                Formatter.__replace_in_comment('@extends', extends, comment.get_value(), namings_to_fix))
 
             comment.set_fixed_value(
                 Formatter.__add_to_comment_if_not_exists('@extends', extends, comment.get_value(), True))
@@ -647,8 +665,8 @@ class Formatter:
         while i < len(comment):
             if comment[i] == '[' and comment[i + 1] == '[':
                 link_start = i + 1
-                link_end = comment.find(i + 1, ']')
-                link = comment[link_start + 1:link_end]
+                link_end = comment.find(']', i + 1)
+                link = comment[link_start:link_end]
                 comment = comment[:link_start] + Formatter.__fix_link(link, namings_to_fix) + comment[link_end:]
                 i = link_end
             i += 1
@@ -661,6 +679,7 @@ class Formatter:
         while i < len(components):
             if components[i] in namings_to_fix:
                 components[i] = namings_to_fix[components[i]]
+            i += 1
         return '.'.join(components)
 
     def __skip_ws_tokens(self, pos):
@@ -681,7 +700,8 @@ class Formatter:
                 pos = Formatter.__find_end_of_params(param, comment)
                 token = Formatter.snake_to_upper_camel_case(token) if upper and Formatter.__is_not_upper_camel_case(
                     token) else token
-                token = Formatter.snake_to_lower_camel_case(token) if not upper and Formatter.__is_not_lower_camel_case(
+                token = Formatter.snake_to_lower_camel_case(
+                    token) if not upper and Formatter.__is_not_lower_camel_case(
                     token) else token
                 comment = comment[:pos] + '\n' + ws + f'  * {param} {token}' + comment[pos:]
 
