@@ -163,7 +163,8 @@ class Lexer:
         return position
 
     def __parse_number_literal(self, code):
-        regex = re.compile("([0-9]*[.])?[0-9]+")
+        regex = re.compile(
+            "(0x([0-9a-fA-F]*_?)*)|((?:\d*[a-zA-Z]*)*)|(\d*(?:(?=\d*)\.\d*)(?:E?(?=[-|\d]*)-?\d*))|(([0-9]*[.])?[0-9]+)|")
         result = regex.search(code, self.__position)
         number_literal = code[self.__position:result.span()[1]]
         self.__column += result.span()[1] - self.__position
